@@ -159,10 +159,11 @@ ENCAtlas::~ENCAtlas()
 }
 
 void ENCAtlas::load(const Projection &in, const Projection &out,
-  qreal deviceRatio, bool hidpi, int layer)
+  qreal deviceRatio, bool hidpi, int style, int layer)
 {
 	Q_UNUSED(in);
 	Q_UNUSED(hidpi);
+	Q_UNUSED(style);
 	Q_UNUSED(layer);
 
 	_tileRatio = deviceRatio;
