@@ -459,7 +459,11 @@ RectC MapData::bounds() const
 void MapData::load()
 {
 	QFile file(_fileName);
-	if (file.open(QIODevice::ReadOnly | QIODevice::Unbuffered))
+
+	if (!file.open(QIODevice::ReadOnly | QIODevice::Unbuffered))
+		qWarning("%s: %s", qUtf8Printable(file.fileName()),
+		  qUtf8Printable(file.errorString()));
+	else
 		readSubFiles(file);
 }
 
