@@ -26,7 +26,7 @@ public:
 	void draw(QPainter *painter, const QRectF &rect, Flags flags);
 
 	void load(const Projection &in, const Projection &out, qreal deviceRatio,
-	  bool hidpi);
+	  bool hidpi, int style, int layer);
 	void unload();
 
 	bool isValid() const {return _valid;}
